use utf8;
package [d2% appname %2d]::Schema;

# Created by DBIx::Class::Schema::Loader
# DO NOT MODIFY THE FIRST PART OF THIS FILE

use strict;
use warnings;

use base 'DBIx::Class::Schema';

__PACKAGE__->load_namespaces;


# Created by DBIx::Class::Schema::Loader v0.07052 @ 2025-01-17 22:29:56
# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:lxoqFoUlG8ltcbtq4b0ehA


# You can replace this text with custom code or comments, and it will be preserved on regeneration
1;
